﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Entity;
using System.Configuration;
using System.Linq.Expressions;

namespace MvcEFCodeFirst.DataAccess
{

  /// <summary>
  /// Repository base class used with DbContext
  /// </summary>
  /// <typeparam name="TContext">Type of DdContext that this repository operates on</typeparam>
  public abstract class Repository<TContext> : IDisposable, IRepository
      where TContext : DbContext, new()
  {
    private TContext context;

    /// <summary>
    /// Create new instance of repository
    /// </summary>
    /// <param name="connectionStringName">Connection string name from .config file</param>
    public Repository(string connectionStringName)
    {
      context = new TContext();
      context.Database.Connection.ConnectionString =
          ConfigurationManager.ConnectionStrings[connectionStringName].ConnectionString;
    }

    private Repository()
    {
    }

    /// <summary>
    /// Dispose repository
    /// </summary>
    public void Dispose()
    {
      if (context != null)
      {
        context.Dispose();
        context = null;
      }
    }


    /// <summary>
    /// Select data from database
    /// </summary>
    /// <typeparam name="TItem">Type of data to select</typeparam>
    /// <returns></returns>
    public IQueryable<TItem> Select<TItem>()
       where TItem : class, new()
    {
      DbSet<TItem> set = context.Set<TItem>();
      return set;
    }

    /// <summary>
    /// Insert new item into database
    /// </summary>
    /// <typeparam name="TItem">Type of item to insert</typeparam>
    /// <param name="item">Item to insert</param>
    /// <returns>Inserted item</returns>
    public TItem Insert<TItem>(TItem item)
        where TItem : class, new()
    {
      DbSet<TItem> set = context.Set<TItem>();
      set.Add(item);
      context.SaveChanges();
      return item;
    }

    /// <summary>
    /// Update an item
    /// </summary>
    /// <typeparam name="TItem">Type of item to update</typeparam>
    /// <param name="item">Item to update</param>
    /// <returns>Updated item</returns>
    public TItem Update<TItem>(TItem item)
        where TItem : class, new()
    {
      DbSet<TItem> set = context.Set<TItem>();
      set.Attach(item);
      context.Entry(item).State = System.Data.EntityState.Modified;
      context.SaveChanges();
      return item;
    }

    /// <summary>
    /// Delete an item
    /// </summary>
    /// <typeparam name="TItem">Type of item to delete</typeparam>
    /// <param name="item">Item to delete</param>
    public void Delete<TItem>(TItem item)
       where TItem : class, new()
    {
      DbSet<TItem> set = context.Set<TItem>();
      var entry = context.Entry(item);
      if (entry != null)
      {
        entry.State = System.Data.EntityState.Deleted;
      }
      else
      {
        set.Attach(item);
      }
      context.Entry(item).State = System.Data.EntityState.Deleted;
      context.SaveChanges();
    }

    public TItem Find<TItem>(params object[] keyValues)
      where TItem : class, new()
    {
      return context.Set<TItem>().Find(keyValues);
    }

    public void SetModified(object item)
    {
      context.Entry(item).State = System.Data.EntityState.Modified;
    }

    public void Save()
    {
      context.SaveChanges();
    }
  }
}
